@@ -27,7 +27,7 @@ class MCCA:
 
     """
 
-    def __init__(self, latent_dims: int, r: float = 1e-7, eps: float = 1e-7):
+    def __init__(self, latent_dims: int, r: float = 0, eps: float = 1e-3):
         """
 
         :param latent_dims: the number of latent dimensions
@@ -51,7 +51,7 @@ class MCCA:
         C = torch.matmul(all_views.T, all_views)
 
         # Get the block covariance matrix placing Xi^TX_i on the diagonal
-        D = torch.block_diag(*[torch.matmul(view.T, view) for view in views])
+        D = torch.block_diag(*[_minimal_regularisation(torch.matmul(view.T, view), self.eps) for view in views])
 
         # In MCCA our eigenvalue problem Cv = lambda Dv
 
@@ -79,7 +79,7 @@ class GCCA:
 
     """
 
-    def __init__(self, latent_dims: int, r: float = 1e-7, eps: float = 1e-7):
+    def __init__(self, latent_dims: int, r: float = 0, eps: float = 1e-3):
         """
 
         :param latent_dims: the number of latent dimensions
@@ -129,7 +129,7 @@ class CCA:
 
     """
 
-    def __init__(self, latent_dims: int, r: float = 1e-7, eps: float = 1e-7):
+    def __init__(self, latent_dims: int, r: float = 0, eps: float = 1e-3):
         """
         :param latent_dims: the number of latent dimensions
         :param r: regularisation as in regularized CCA. Makes the problem well posed when batch size is similar to the number of latent dimensions
@@ -160,14 +160,15 @@ class CCA:
                                                                                                    dtype=torch.double,
                                                                                                    device=H2.device).float()
 
-        SigmaHat11RootInv = _compute_matrix_power(_minimal_regularisation(SigmaHat11, self.eps), -0.5)
-        SigmaHat22RootInv = _compute_matrix_power(_minimal_regularisation(SigmaHat22, self.eps), -0.5)
+        # performs the inverse square root of the covariance matrices by the cholesky decomposition. This is more stable than using SVD
+        SigmaHat11RootInv = torch.linalg.inv(torch.linalg.cholesky(_minimal_regularisation(SigmaHat11, self.eps)))
+        SigmaHat22RootInv = torch.linalg.inv(torch.linalg.cholesky(_minimal_regularisation(SigmaHat22, self.eps)))
 
         Tval = torch.matmul(torch.matmul(SigmaHat11RootInv,
                                          SigmaHat12), SigmaHat22RootInv)
 
         trace_TT = torch.matmul(Tval.t(), Tval)
-        eigvals = torch.real(torch.linalg.eigvals(_minimal_regularisation(trace_TT, self.eps)))
+        eigvals = torch.real(torch.linalg.eigvals(trace_TT))
         eigvals = eigvals[torch.gt(eigvals, self.eps)]
         corr = torch.sum(torch.sqrt(eigvals))
         return -corr
@@ -179,7 +180,7 @@ class TCCA:
 
     """
 
-    def __init__(self, latent_dims: int, r: float = 1e-7, eps: float = 1e-7):
+    def __init__(self, latent_dims: int, r: float = 0, eps: float = 1e-3):
         """
 
         :param latent_dims: the number of latent dimensions
