@@ -1,7 +1,7 @@
 import copy
 import warnings
 from abc import abstractmethod
-from typing import Union, List
+from typing import Union, Iterable
 
 import numpy as np
 
@@ -16,15 +16,19 @@ class _Iterative(_CCA_Base):
 
     """
 
-    def __init__(self, latent_dims: int = 1, scale: bool = True, centre=True, copy_data=True, deflation='cca',
+    def __init__(self, latent_dims: int = 1, scale: bool = True, centre=True, copy_data=True, random_state=None,
+                 deflation='cca',
                  max_iter: int = 100,
                  generalized: bool = False,
                  initialization: str = 'unregularized', tol: float = 1e-9):
         """
         Constructor for _Iterative
 
-        :param latent_dims: number of latent dimensions
-        :param scale: scale data by column variances before optimisation
+        :param latent_dims: number of latent dimensions to fit
+        :param scale: normalize variance in each column before fitting
+        :param centre: demean data by column before fitting (and before transforming out of sample
+        :param copy_data: If True, X will be copied; else, it may be overwritten
+        :param random_state: Pass for reproducible output across multiple function calls
         :param deflation: the type of deflation.
         :param max_iter: the maximum number of iterations to perform in the inner optimization loop
         :param generalized: use auxiliary variables (required for >2 views)
@@ -37,6 +41,7 @@ class _Iterative(_CCA_Base):
         self.initialization = initialization
         self.tol = tol
         self.deflation = deflation
+        self.random_state = random_state
 
     def fit(self, *views: np.ndarray, ):
         """
@@ -44,16 +49,16 @@ class _Iterative(_CCA_Base):
 
         :param views: numpy arrays with the same number of rows (samples) separated by commas
         """
-        self.set_loop_params()
-        train_views = self.centre_scale(*views)
+        self._set_loop_params()
+        train_views = self._centre_scale(*views)
         n = train_views[0].shape[0]
         p = [view.shape[1] for view in train_views]
-        # list of d: p x k
-        self.weights_list = [np.zeros((p_, self.latent_dims)) for p_ in p]
-        self.loading_list = [np.zeros((p_, self.latent_dims)) for p_ in p]
+        # List of d: p x k
+        self.weights = [np.zeros((p_, self.latent_dims)) for p_ in p]
+        self.loadings = [np.zeros((p_, self.latent_dims)) for p_ in p]
 
-        # list of d: n x k
-        self.score_list = [np.zeros((n, self.latent_dims)) for _ in train_views]
+        # List of d: n x k
+        self.scores = [np.zeros((n, self.latent_dims)) for _ in train_views]
 
         residuals = copy.deepcopy(list(train_views))
 
@@ -62,16 +67,16 @@ class _Iterative(_CCA_Base):
         for k in range(self.latent_dims):
             self.loop = self.loop.fit(*residuals)
             for i, residual in enumerate(residuals):
-                self.weights_list[i][:, k] = self.loop.weights[i]
-                self.score_list[i][:, k] = self.loop.scores[i]
-                self.loading_list[i][:, k] = np.dot(self.loop.scores[i], residual)
+                self.weights[i][:, k] = self.loop.weights[i]
+                self.scores[i][:, k] = self.loop.scores[i]
+                self.loadings[i][:, k] = np.dot(self.loop.scores[i], residual)
                 # TODO This is CCA deflation (https://ars.els-cdn.com/content/image/1-s2.0-S0006322319319183-mmc1.pdf)
-                residuals[i] = self.deflate(residuals[i], self.score_list[i][:, k], self.weights_list[i][:, k])
+                residuals[i] = self._deflate(residuals[i], self.scores[i][:, k], self.weights[i][:, k])
             self.objective.append(self.loop.track_objective)
         self.train_correlations = self.predict_corr(*views)
         return self
 
-    def deflate(self, residual, score, loading):
+    def _deflate(self, residual, score, loading):
         """
         Deflate view residual by CCA deflation (https://ars.els-cdn.com/content/image/1-s2.0-S0006322319319183-mmc1.pdf)
 
@@ -85,12 +90,12 @@ class _Iterative(_CCA_Base):
             return residual - np.outer(score, loading)
 
     @abstractmethod
-    def set_loop_params(self):
+    def _set_loop_params(self):
         """
         Sets up the inner optimization loop for the method. By default uses the PLS inner loop.
         """
         self.loop = PLSInnerLoop(max_iter=self.max_iter, generalized=self.generalized,
-                                 initialization=self.initialization)
+                                 initialization=self.initialization, random_state=self.random_state)
 
 
 class PLS(_Iterative):
@@ -108,28 +113,33 @@ class PLS(_Iterative):
     >>> model.fit(X1,X2)
     """
 
-    def __init__(self, latent_dims: int = 1, scale: bool = True, centre=True, copy_data=True, deflation='cca',
+    def __init__(self, latent_dims: int = 1, scale: bool = True, centre=True, copy_data=True, random_state=None,
+                 deflation='cca',
                  max_iter: int = 100,
                  generalized: bool = False,
                  initialization: str = 'unregularized', tol: float = 1e-9):
         """
         Constructor for PLS
 
-        :param latent_dims: number of latent dimensions
-        :param scale: scale data by column variances before optimisation
+        :param latent_dims: number of latent dimensions to fit
+        :param scale: normalize variance in each column before fitting
+        :param centre: demean data by column before fitting (and before transforming out of sample
+        :param copy_data: If True, X will be copied; else, it may be overwritten
+        :param random_state: Pass for reproducible output across multiple function calls
         :param deflation: the type of deflation.
         :param max_iter: the maximum number of iterations to perform in the inner optimization loop
         :param generalized: use auxiliary variables (required for >2 views)
         :param initialization: intialization for optimisation. 'unregularized' uses CCA or PLS solution,'random' uses random initialization,'uniform' uses uniform initialization of weights and scores
         :param tol: if the cosine similarity of the weights between subsequent iterations is greater than 1-tol the loop is considered converged
         """
-        super().__init__(latent_dims=latent_dims, scale=scale, centre=centre, deflation=deflation, max_iter=max_iter,
+        super().__init__(latent_dims=latent_dims, scale=scale, centre=centre, copy_data=copy_data, deflation=deflation,
+                         max_iter=max_iter,
                          generalized=generalized,
-                         initialization=initialization, tol=tol)
+                         initialization=initialization, tol=tol, random_state=random_state)
 
-    def set_loop_params(self):
+    def _set_loop_params(self):
         self.loop = PLSInnerLoop(max_iter=self.max_iter, generalized=self.generalized,
-                                 initialization=self.initialization, tol=self.tol)
+                                 initialization=self.initialization, tol=self.tol, random_state=self.random_state)
 
 
 class ElasticCCA(_Iterative):
@@ -149,19 +159,23 @@ class ElasticCCA(_Iterative):
     >>> model.fit(X1,X2)
     """
 
-    def __init__(self, latent_dims: int = 1, scale: bool = True, centre=True, copy_data=True, deflation='cca',
+    def __init__(self, latent_dims: int = 1, scale: bool = True, centre=True, copy_data=True, random_state=None,
+                 deflation='cca',
                  max_iter: int = 100,
                  generalized: bool = False,
                  initialization: str = 'unregularized', tol: float = 1e-9,
-                 c: Union[List[float], float] = None,
-                 l1_ratio: Union[List[float], float] = None,
+                 c: Union[Iterable[float], float] = None,
+                 l1_ratio: Union[Iterable[float], float] = None,
                  constrained: bool = False, stochastic=False,
-                 positive: Union[List[bool], bool] = None):
+                 positive: Union[Iterable[bool], bool] = None):
         """
         Constructor for ElasticCCA
 
-        :param latent_dims: number of latent dimensions
-        :param scale: scale data by column variances before optimisation
+        :param latent_dims: number of latent dimensions to fit
+        :param scale: normalize variance in each column before fitting
+        :param centre: demean data by column before fitting (and before transforming out of sample
+        :param copy_data: If True, X will be copied; else, it may be overwritten
+        :param random_state: Pass for reproducible output across multiple function calls
         :param deflation: the type of deflation.
         :param max_iter: the maximum number of iterations to perform in the inner optimization loop
         :param generalized: use auxiliary variables (required for >2 views)
@@ -185,13 +199,14 @@ class ElasticCCA(_Iterative):
         super().__init__(latent_dims=latent_dims, scale=scale, centre=centre, copy_data=copy_data, deflation=deflation,
                          max_iter=max_iter,
                          generalized=generalized,
-                         initialization=initialization, tol=tol)
+                         initialization=initialization, tol=tol, random_state=random_state
+                         )
 
-    def set_loop_params(self):
+    def _set_loop_params(self):
         self.loop = ElasticInnerLoop(max_iter=self.max_iter, c=self.c, l1_ratio=self.l1_ratio,
                                      generalized=self.generalized, initialization=self.initialization,
                                      tol=self.tol, constrained=self.constrained,
-                                     stochastic=self.stochastic, positive=self.positive)
+                                     stochastic=self.stochastic, positive=self.positive, random_state=self.random_state)
 
 
 class CCA_ALS(ElasticCCA):
@@ -211,18 +226,31 @@ class CCA_ALS(ElasticCCA):
     >>> model.fit(X1,X2)
     """
 
-    def __init__(self, latent_dims: int = 1, scale: bool = True, centre=True, copy_data=True, max_iter: int = 100,
+    def __init__(self, latent_dims: int = 1, scale: bool = True, centre=True, copy_data=True, random_state=None,
+                 max_iter: int = 100,
                  generalized: bool = False,
                  initialization: str = 'random', tol: float = 1e-9, stochastic=True,
-                 positive: Union[List[bool], bool] = None):
+                 positive: Union[Iterable[bool], bool] = None):
         """
         Constructor for CCA_ALS
 
+        :param latent_dims: number of latent dimensions to fit
+        :param scale: normalize variance in each column before fitting
+        :param centre: demean data by column before fitting (and before transforming out of sample
+        :param copy_data: If True, X will be copied; else, it may be overwritten
+        :param random_state: Pass for reproducible output across multiple function calls
+        :param max_iter: the maximum number of iterations to perform in the inner optimization loop
+        :param generalized: use auxiliary variables (required for >2 views)
+        :param initialization: intialization for optimisation. 'unregularized' uses CCA or PLS solution,'random' uses random initialization,'uniform' uses uniform initialization of weights and scores
+        :param tol: if the cosine similarity of the weights between subsequent iterations is greater than 1-tol the loop is considered converged
+        :param stochastic: use stochastic regression optimisers for subproblems
+        :param positive: constrain model weights to be positive
         """
 
         super().__init__(latent_dims=latent_dims, max_iter=max_iter, generalized=generalized,
-                         initialization=initialization, tol=tol, constrained=False, stochastic=stochastic, scale=scale,
-                         positive=positive)
+                         initialization=initialization, tol=tol, constrained=False, stochastic=stochastic,
+                         centre=centre, copy_data=copy_data, scale=scale,
+                         positive=positive, random_state=random_state)
 
 
 class SCCA(ElasticCCA):
@@ -242,20 +270,32 @@ class SCCA(ElasticCCA):
     >>> model.fit(X1,X2)
     """
 
-    def __init__(self, latent_dims: int = 1, scale: bool = True, centre=True, copy_data=True,
-                 c: Union[List[float], float] = None,
+    def __init__(self, latent_dims: int = 1, scale: bool = True, centre=True, copy_data=True, random_state=None,
+                 c: Union[Iterable[float], float] = None,
                  max_iter: int = 100,
                  generalized: bool = False,
                  initialization: str = 'unregularized', tol: float = 1e-9, stochastic=False,
-                 positive: Union[List[bool], bool] = None):
+                 positive: Union[Iterable[bool], bool] = None):
         """
         Constructor for SCCA
 
+        :param latent_dims: number of latent dimensions to fit
+        :param scale: normalize variance in each column before fitting
+        :param centre: demean data by column before fitting (and before transforming out of sample
+        :param copy_data: If True, X will be copied; else, it may be overwritten
+        :param random_state: Pass for reproducible output across multiple function calls
+        :param max_iter: the maximum number of iterations to perform in the inner optimization loop
+        :param generalized: use auxiliary variables (required for >2 views)
+        :param initialization: intialization for optimisation. 'unregularized' uses CCA or PLS solution,'random' uses random initialization,'uniform' uses uniform initialization of weights and scores
+        :param tol: if the cosine similarity of the weights between subsequent iterations is greater than 1-tol the loop is considered converged
+        :param c: lasso alpha
+        :param stochastic: use stochastic regression optimisers for subproblems
+        :param positive: constrain model weights to be positive
         """
         super().__init__(latent_dims=latent_dims, scale=scale, centre=centre, copy_data=copy_data, max_iter=max_iter,
                          generalized=generalized,
                          initialization=initialization, tol=tol, c=c, l1_ratio=1, constrained=False,
-                         stochastic=stochastic, positive=positive)
+                         stochastic=stochastic, positive=positive, random_state=random_state)
 
 
 class PMD(_Iterative):
@@ -275,14 +315,19 @@ class PMD(_Iterative):
     >>> model.fit(X1,X2)
     """
 
-    def __init__(self, latent_dims: int = 1, scale: bool = True, centre=True, copy_data=True,
-                 c: Union[List[float], float] = None,
+    def __init__(self, latent_dims: int = 1, scale: bool = True, centre=True, copy_data=True, random_state=None,
+                 c: Union[Iterable[float], float] = None,
                  max_iter: int = 100,
                  generalized: bool = False, initialization: str = 'unregularized', tol: float = 1e-9,
-                 positive: Union[List[bool], bool] = None):
+                 positive: Union[Iterable[bool], bool] = None):
         """
         Constructor for PMD
 
+        :param latent_dims: number of latent dimensions to fit
+        :param scale: normalize variance in each column before fitting
+        :param centre: demean data by column before fitting (and before transforming out of sample
+        :param copy_data: If True, X will be copied; else, it may be overwritten
+        :param random_state: Pass for reproducible output across multiple function calls
         :param c: l1 regularisation parameter between 1 and sqrt(number of features) for each view
         :param positive: constrain model weights to be positive
         """
@@ -290,11 +335,12 @@ class PMD(_Iterative):
         self.positive = positive
         super().__init__(latent_dims=latent_dims, scale=scale, centre=centre, copy_data=copy_data, max_iter=max_iter,
                          generalized=generalized,
-                         initialization=initialization, tol=tol)
+                         initialization=initialization, tol=tol, random_state=random_state)
 
-    def set_loop_params(self):
+    def _set_loop_params(self):
         self.loop = PMDInnerLoop(max_iter=self.max_iter, c=self.c, generalized=self.generalized,
-                                 initialization=self.initialization, tol=self.tol, positive=self.positive)
+                                 initialization=self.initialization, tol=self.tol, positive=self.positive,
+                                 random_state=self.random_state)
 
 
 class ParkhomenkoCCA(_Iterative):
@@ -314,24 +360,30 @@ class ParkhomenkoCCA(_Iterative):
     >>> model.fit(X1,X2)
     """
 
-    def __init__(self, latent_dims: int = 1, scale: bool = True, centre=True, copy_data=True,
-                 c: Union[List[float], float] = None,
+    def __init__(self, latent_dims: int = 1, scale: bool = True, centre=True, copy_data=True, random_state=None,
+                 c: Union[Iterable[float], float] = None,
                  max_iter: int = 100,
                  generalized: bool = False, initialization: str = 'unregularized', tol: float = 1e-9):
         """
         Constructor for ParkhomenkoCCA
 
+        :param latent_dims: number of latent dimensions to fit
+        :param scale: normalize variance in each column before fitting
+        :param centre: demean data by column before fitting (and before transforming out of sample
+        :param copy_data: If True, X will be copied; else, it may be overwritten
+        :param random_state: Pass for reproducible output across multiple function calls
         :param c: l1 regularisation parameter
         """
         self.c = c
         super().__init__(latent_dims=latent_dims, scale=scale, centre=centre, copy_data=copy_data, max_iter=max_iter,
                          generalized=generalized,
-                         initialization=initialization, tol=tol)
+                         initialization=initialization, tol=tol, random_state=random_state)
 
-    def set_loop_params(self):
+    def _set_loop_params(self):
         self.loop = ParkhomenkoInnerLoop(max_iter=self.max_iter, c=self.c,
                                          generalized=self.generalized,
-                                         initialization=self.initialization, tol=self.tol)
+                                         initialization=self.initialization, tol=self.tol,
+                                         random_state=self.random_state)
 
 
 class SCCA_ADMM(_Iterative):
@@ -351,17 +403,21 @@ class SCCA_ADMM(_Iterative):
     >>> model.fit(X1,X2)
     """
 
-    def __init__(self, latent_dims: int = 1, scale: bool = True, centre=True, copy_data=True,
-                 c: Union[List[float], float] = None,
-                 mu: Union[List[float], float] = None,
-                 lam: Union[List[float], float] = None,
-                 eta: Union[List[float], float] = None,
+    def __init__(self, latent_dims: int = 1, scale: bool = True, centre=True, copy_data=True, random_state=None,
+                 c: Union[Iterable[float], float] = None,
+                 mu: Union[Iterable[float], float] = None,
+                 lam: Union[Iterable[float], float] = None,
+                 eta: Union[Iterable[float], float] = None,
                  max_iter: int = 100,
                  generalized: bool = False, initialization: str = 'unregularized', tol: float = 1e-9):
         """
         Constructor for SCCA_ADMM
 
-
+        :param latent_dims: number of latent dimensions to fit
+        :param scale: normalize variance in each column before fitting
+        :param centre: demean data by column before fitting (and before transforming out of sample
+        :param copy_data: If True, X will be copied; else, it may be overwritten
+        :param random_state: Pass for reproducible output across multiple function calls
         :param c: l1 regularisation parameter
         :param mu:
         :param lam:
@@ -373,12 +429,12 @@ class SCCA_ADMM(_Iterative):
         self.eta = eta
         super().__init__(latent_dims=latent_dims, scale=scale, centre=centre, copy_data=copy_data, max_iter=max_iter,
                          generalized=generalized,
-                         initialization=initialization, tol=tol)
+                         initialization=initialization, tol=tol, random_state=random_state)
 
-    def set_loop_params(self):
+    def _set_loop_params(self):
         self.loop = ADMMInnerLoop(max_iter=self.max_iter, c=self.c, mu=self.mu, lam=self.lam,
                                   eta=self.eta, generalized=self.generalized,
-                                  initialization=self.initialization, tol=self.tol)
+                                  initialization=self.initialization, tol=self.tol, random_state=self.random_state)
 
 
 class SpanCCA(_Iterative):
@@ -394,20 +450,37 @@ class SpanCCA(_Iterative):
     def __init__(self, latent_dims: int = 1, scale: bool = True, centre=True, copy_data=True, max_iter: int = 100,
                  generalized: bool = False,
                  initialization: str = 'uniform', tol: float = 1e-9, regularisation='l0',
-                 c: Union[List[Union[float, int]], Union[float, int]] = None, rank=1,
-                 positive: Union[List[bool], bool] = None):
+                 c: Union[Iterable[Union[float, int]], Union[float, int]] = None, rank=1,
+                 positive: Union[Iterable[bool], bool] = None, random_state=None):
+        """
+
+        :param latent_dims: number of latent dimensions to fit
+        :param scale: normalize variance in each column before fitting
+        :param centre: demean data by column before fitting (and before transforming out of sample
+        :param copy_data: If True, X will be copied; else, it may be overwritten
+        :param random_state: Pass for reproducible output across multiple function calls
+        :param max_iter:
+        :param generalized:
+        :param initialization:
+        :param tol:
+        :param regularisation:
+        :param c:
+        :param rank:
+        :param positive:
+        """
         super().__init__(latent_dims=latent_dims, scale=scale, centre=centre, copy_data=copy_data, max_iter=max_iter,
                          generalized=generalized,
-                         initialization=initialization, tol=tol)
+                         initialization=initialization, tol=tol, random_state=random_state)
         self.c = c
         self.regularisation = regularisation
         self.rank = rank
         self.positive = positive
 
-    def set_loop_params(self):
+    def _set_loop_params(self):
         self.loop = SpanCCAInnerLoop(max_iter=self.max_iter, c=self.c, generalized=self.generalized,
                                      initialization=self.initialization, tol=self.tol,
-                                     regularisation=self.regularisation, rank=self.rank, positive=self.positive)
+                                     regularisation=self.regularisation, rank=self.rank, positive=self.positive,
+                                     random_state=self.random_state)
 
 
 class SWCCA(_Iterative):
@@ -421,21 +494,40 @@ class SWCCA(_Iterative):
 
     """
 
-    def __init__(self, latent_dims: int = 1, scale: bool = True, centre=True, copy_data=True, max_iter: int = 500,
+    def __init__(self, latent_dims: int = 1, scale: bool = True, centre=True, copy_data=True, random_state=None,
+                 max_iter: int = 500,
                  generalized: bool = False,
                  initialization: str = 'uniform', tol: float = 1e-9, regularisation='l0',
-                 c: Union[List[Union[float, int]], Union[float, int]] = None, sample_support=None,
-                 positive: Union[List[bool], bool] = None):
+                 c: Union[Iterable[Union[float, int]], Union[float, int]] = None, sample_support=None,
+                 positive: Union[Iterable[bool], bool] = None):
+        """
+
+        :param latent_dims: number of latent dimensions to fit
+        :param scale: normalize variance in each column before fitting
+        :param centre: demean data by column before fitting (and before transforming out of sample
+        :param copy_data: If True, X will be copied; else, it may be overwritten
+        :param random_state: Pass for reproducible output across multiple function calls
+        :param max_iter:
+        :param generalized:
+        :param initialization:
+        :param tol:
+        :param regularisation:
+        :param c:
+        :param sample_support:
+        :param positive:
+        """
+
         self.c = c
         self.sample_support = sample_support
         self.regularisation = regularisation
         self.positive = positive
         super().__init__(latent_dims=latent_dims, scale=scale, centre=centre, copy_data=copy_data, max_iter=max_iter,
                          generalized=generalized,
-                         initialization=initialization, tol=tol)
+                         initialization=initialization, tol=tol, random_state=random_state)
 
-    def set_loop_params(self):
+    def _set_loop_params(self):
         self.loop = SWCCAInnerLoop(max_iter=self.max_iter, generalized=self.generalized,
                                    initialization=self.initialization, tol=self.tol, regularisation=self.regularisation,
                                    c=self.c,
-                                   sample_support=self.sample_support, positive=self.positive)
+                                   sample_support=self.sample_support, positive=self.positive,
+                                   random_state=self.random_state)
