@@ -17,6 +17,10 @@ class TestData(TestCase):
     def tearDown(self):
         pass
 
+    def test_data_rand(self):
+        (x, y), true_feats = generate_covariance_data(1000, [10, 11], 1, [0.5, 0.5], correlation=0.5,
+                                                      structure='random')
+
     def test_data_gen(self):
         (x, y, z), true_feats = generate_covariance_data(1000, [10, 11, 12], 1, [0.5, 0.5, 0.5], correlation=0.5,
                                                          structure=['identity', 'identity', 'identity'])
