@@ -3,21 +3,21 @@ from unittest import TestCase
 
 import numpy as np
 import scipy.sparse as sp
+from sklearn.utils.validation import check_random_state
 
 from cca_zoo.models import CCA, PLS, CCA_ALS, SCCA, PMD, ElasticCCA, rCCA, KCCA, KTCCA, MCCA, GCCA, TCCA, SCCA_ADMM, \
     SpanCCA, SWCCA
 
-np.random.seed(123)
-
 
 class TestModels(TestCase):
 
     def setUp(self):
-        self.X = np.random.rand(500, 20)
-        self.Y = np.random.rand(500, 21)
-        self.Z = np.random.rand(500, 22)
-        self.X_sp = sp.random(1000, 20, density=0.1)
-        self.Y_sp = sp.random(1000, 21, density=0.1)
+        self.rng = check_random_state(0)
+        self.X = self.rng.rand(500, 20)
+        self.Y = self.rng.rand(500, 21)
+        self.Z = self.rng.rand(500, 22)
+        self.X_sp = sp.random(1000, 20, density=0.1, random_state=self.rng)
+        self.Y_sp = sp.random(1000, 21, density=0.1, random_state=self.rng)
 
     def tearDown(self):
         pass
@@ -26,7 +26,7 @@ class TestModels(TestCase):
         # Tests unregularized CCA methods. The idea is that all of these should give the same result.
         latent_dims = 1
         wrap_cca = CCA(latent_dims=latent_dims).fit(self.X, self.Y)
-        wrap_iter = CCA_ALS(latent_dims=latent_dims, tol=1e-9).fit(self.X, self.Y)
+        wrap_iter = CCA_ALS(latent_dims=latent_dims, tol=1e-9, random_state=self.rng).fit(self.X, self.Y)
         wrap_gcca = GCCA(latent_dims=latent_dims).fit(self.X, self.Y)
         wrap_mcca = MCCA(latent_dims=latent_dims).fit(self.X, self.Y)
         wrap_kcca = KCCA(latent_dims=latent_dims).fit(self.X, self.Y)
@@ -36,10 +36,10 @@ class TestModels(TestCase):
         corr_mcca = wrap_mcca.train_correlations[0, 1]
         corr_kcca = wrap_kcca.train_correlations[0, 1]
         # Check the score outputs are the right shape
-        self.assertTrue(wrap_iter.score_list[0].shape == (self.X.shape[0], latent_dims))
-        self.assertTrue(wrap_gcca.score_list[0].shape == (self.X.shape[0], latent_dims))
-        self.assertTrue(wrap_mcca.score_list[0].shape == (self.X.shape[0], latent_dims))
-        self.assertTrue(wrap_kcca.score_list[0].shape == (self.X.shape[0], latent_dims))
+        self.assertTrue(wrap_iter.scores[0].shape == (self.X.shape[0], latent_dims))
+        self.assertTrue(wrap_gcca.scores[0].shape == (self.X.shape[0], latent_dims))
+        self.assertTrue(wrap_mcca.scores[0].shape == (self.X.shape[0], latent_dims))
+        self.assertTrue(wrap_kcca.scores[0].shape == (self.X.shape[0], latent_dims))
         # Check the correlations from each unregularized method are the same
         self.assertIsNone(np.testing.assert_array_almost_equal(corr_cca, corr_iter, decimal=2))
         self.assertIsNone(np.testing.assert_array_almost_equal(corr_iter, corr_mcca, decimal=2))
@@ -60,10 +60,10 @@ class TestModels(TestCase):
         corr_mcca = wrap_mcca.train_correlations[0, 1]
         corr_kcca = wrap_kcca.train_correlations[0, 1]
         # Check the score outputs are the right shape
-        self.assertTrue(wrap_iter.score_list[0].shape == (self.X_sp.shape[0], latent_dims))
-        self.assertTrue(wrap_gcca.score_list[0].shape == (self.X_sp.shape[0], latent_dims))
-        self.assertTrue(wrap_mcca.score_list[0].shape == (self.X_sp.shape[0], latent_dims))
-        self.assertTrue(wrap_kcca.score_list[0].shape == (self.X_sp.shape[0], latent_dims))
+        self.assertTrue(wrap_iter.scores[0].shape == (self.X_sp.shape[0], latent_dims))
+        self.assertTrue(wrap_gcca.scores[0].shape == (self.X_sp.shape[0], latent_dims))
+        self.assertTrue(wrap_mcca.scores[0].shape == (self.X_sp.shape[0], latent_dims))
+        self.assertTrue(wrap_kcca.scores[0].shape == (self.X_sp.shape[0], latent_dims))
         # Check the correlations from each unregularized method are the same
         self.assertIsNone(np.testing.assert_array_almost_equal(corr_cca, corr_iter, decimal=2))
         self.assertIsNone(np.testing.assert_array_almost_equal(corr_iter, corr_mcca, decimal=2))
@@ -85,10 +85,10 @@ class TestModels(TestCase):
         corr_mcca = wrap_mcca.train_correlations[:, :, 0]
         corr_kcca = wrap_kcca.train_correlations[:, :, 0]
         # Check the score outputs are the right shape
-        self.assertTrue(wrap_iter.score_list[0].shape == (self.X.shape[0], latent_dims))
-        self.assertTrue(wrap_gcca.score_list[0].shape == (self.X.shape[0], latent_dims))
-        self.assertTrue(wrap_mcca.score_list[0].shape == (self.X.shape[0], latent_dims))
-        self.assertTrue(wrap_kcca.score_list[0].shape == (self.X.shape[0], latent_dims))
+        self.assertTrue(wrap_iter.scores[0].shape == (self.X.shape[0], latent_dims))
+        self.assertTrue(wrap_gcca.scores[0].shape == (self.X.shape[0], latent_dims))
+        self.assertTrue(wrap_mcca.scores[0].shape == (self.X.shape[0], latent_dims))
+        self.assertTrue(wrap_kcca.scores[0].shape == (self.X.shape[0], latent_dims))
         # Check the correlations from each unregularized method are the same
         self.assertIsNone(np.testing.assert_array_almost_equal(corr_cca, corr_iter, decimal=1))
         self.assertIsNone(np.testing.assert_array_almost_equal(corr_cca, corr_mcca, decimal=2))
@@ -130,18 +130,18 @@ class TestModels(TestCase):
         c1 = [1, 3]
         c2 = [1, 3]
         param_candidates = {'c': list(itertools.product(c1, c2))}
-        wrap_pmd = PMD(latent_dims=latent_dims).gridsearch_fit(self.X, self.Y,
-                                                               param_candidates=param_candidates,
-                                                               verbose=True, plot=True)
+        wrap_pmd = PMD(latent_dims=latent_dims, random_state=self.rng).gridsearch_fit(self.X, self.Y,
+                                                                                      param_candidates=param_candidates,
+                                                                                      verbose=True, plot=True)
         c1 = [1e-4, 1e-5]
         c2 = [1e-4, 1e-5]
         param_candidates = {'c': list(itertools.product(c1, c2))}
-        wrap_scca = SCCA(latent_dims=latent_dims).gridsearch_fit(self.X, self.Y,
-                                                                 param_candidates=param_candidates,
-                                                                 verbose=True)
-        wrap_elastic = ElasticCCA(latent_dims=latent_dims).gridsearch_fit(self.X, self.Y,
-                                                                          param_candidates=param_candidates,
-                                                                          verbose=True)
+        wrap_scca = SCCA(latent_dims=latent_dims, random_state=self.rng).gridsearch_fit(self.X, self.Y,
+                                                                                        param_candidates=param_candidates,
+                                                                                        verbose=True)
+        wrap_elastic = ElasticCCA(latent_dims=latent_dims, random_state=self.rng).gridsearch_fit(self.X, self.Y,
+                                                                                                 param_candidates=param_candidates,
+                                                                                                 verbose=True)
         corr_pmd = wrap_pmd.train_correlations[0, 1]
         corr_scca = wrap_scca.train_correlations[0, 1]
         corr_elastic = wrap_elastic.train_correlations[0, 1]
@@ -189,9 +189,9 @@ class TestModels(TestCase):
     def test_l0(self):
         wrap_span_cca = SpanCCA(latent_dims=1, regularisation='l0', c=[2, 2]).fit(self.X, self.Y)
         wrap_swcca = SWCCA(latent_dims=1, c=[2, 2], sample_support=5).fit(self.X, self.Y)
-        self.assertEqual((np.abs(wrap_span_cca.weights_list[0]) > 1e-5).sum(), 2)
-        self.assertEqual((np.abs(wrap_span_cca.weights_list[1]) > 1e-5).sum(), 2)
-        self.assertEqual((np.abs(wrap_swcca.weights_list[0]) > 1e-5).sum(), 2)
-        self.assertEqual((np.abs(wrap_swcca.weights_list[1]) > 1e-5).sum(), 2)
+        self.assertEqual((np.abs(wrap_span_cca.weights[0]) > 1e-5).sum(), 2)
+        self.assertEqual((np.abs(wrap_span_cca.weights[1]) > 1e-5).sum(), 2)
+        self.assertEqual((np.abs(wrap_swcca.weights[0]) > 1e-5).sum(), 2)
+        self.assertEqual((np.abs(wrap_swcca.weights[1]) > 1e-5).sum(), 2)
         self.assertEqual((np.abs(wrap_swcca.loop.sample_weights) > 1e-5).sum(), 5)
         print()
